from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from gavel.config.settings import DB_CONNECTION
from gavel.config.settings import DBMS

__ENGINE__ = None


def get_engine():
    global __ENGINE__
    if __ENGINE__ is None:
        cred = DB_CONNECTION.get("user", "")
        if cred:
            if "password" in DB_CONNECTION:
                cred = "{user}:{password}".format(**DB_CONNECTION)
            cred += "@"

        location = DB_CONNECTION.get("host", "")
        port = DB_CONNECTION.get("port")
        if port:
            location += ":" + port
        __ENGINE__ = create_engine(
            "{dbms}://{cred}{location}/{database}".format(
                dbms=DBMS, cred=cred, location=location, **DB_CONNECTION
            )
        )
    return __ENGINE__


def with_session(wrapped_function):
    def inside(*args, **kwargs):
        if "session" not in kwargs:
            engine = get_engine()
            Session = sessionmaker(bind=engine)
            session = Session()
            try:
                return wrapped_function(*args, session=session, **kwargs)
            except:
                session.rollback()
                raise
            finally:
                session.close()
        else:
            return wrapped_function(*args, **kwargs)

    return inside


def get_or_create(session, cls, *args, **kwargs):
    obj = get_or_None(session, cls, *args, **kwargs)
    created = False
    if not obj:
        obj = cls(*args, **kwargs)
        session.add(obj)
        created = True
    return obj, created


def get_or_None(session, cls, *args, **kwargs):
    return session.query(cls).filter_by(**kwargs).first()



from gavel.dialects.base.parser import LogicParser
from gavel.dialects.base.parser import Parseable
from gavel.dialects.base.parser import ProblemParser
from gavel.logic import logic
from gavel.logic.problem import Problem
from gavel.logic.problem import Sentence


class DBProblemParser(ProblemParser):
    logic_parser_cls = LogicParser

    def __init__(self, *args, **kwargs):
        super(DBProblemParser, self).__init__(*args, **kwargs)
        self.logic_parser = LogicParser()

    def parse(self, structure: Parseable, *args, **kwargs):
        premises = [self.logic_parser.parse(s) for s in structure.premises]
        conjecture = self.logic_parser.parse(structure.conjecture)
        return Problem(premises, conjecture)


class DBLogicParser(LogicParser):
    def _parse_rec(self, obj, *args, **kwargs):
        if isinstance(obj, str):
            return obj
        meth = getattr(self, "parse_%s" % obj.get("type"), None)

        if meth is None:
            raise Exception(
                "Parser '{name}' not found for {cls}".format(
                    name=obj.get("type"), cls=obj
                )
            )
        return meth(obj)

    def parse_quantifier(self, quantifier: dict):
        if quantifier.get("type") == "existential":
            q = logic.Quantifier.EXISTENTIAL
        elif quantifier.get("type") == "universial":
            q = logic.Quantifier.UNIVERSAL
        else:
            raise NotImplementedError
        return dict(type="quantifier", quantifier=q)

    def parse_formula_role(self, role: dict):
        return getattr(logic.FormulaRole, role["formula_role"].upper())

    def parse_binary_connective(self, connective: dict):
        return getattr(logic.BinaryConnective, connective["binary_connective"].upper())

    def parse_defined_predicate(self, predicate: dict):
        return getattr(logic.DefinedPredicate, predicate["defined_predicate"].upper())

    def parse_unary_connective(self, connective: dict):
        return getattr(logic.UnaryConnective, connective["unary_connective"].upper())

    def parse_unary_formula(self, formula: dict):
        return logic.UnaryFormula(
            formula=self._parse_rec(formula["formula"]),
            connective=self._parse_rec(formula["connective"]),
        )

    def parse_quantified_formula(self, formula: dict):
        return logic.QuantifiedFormula(
            formula=self._parse_rec(formula["formula"]),
            quantifier=self._parse_rec(formula["quantifier"]),
            variables=[self._parse_rec(v) for v in formula["quantifier"]],
        )

    def parse_annotated_formula(self, anno: dict):
        return logic.AnnotatedFormula(
            formula=self._parse_rec(anno["formula"]),
            name=anno["name"],
            role=self._parse_rec(anno["role"]),
            logic=anno["logic"],
        )

    def parse_binary_formula(self, formula: dict):
        return logic.BinaryFormula(
            left=self._parse_rec(formula["left"]),
            right=self._parse_rec(formula["right"]),
            operator=self._parse_rec(formula["connective"]),
        )

    def parse_functor_expression(self, expression: logic.FunctorExpression):
        return logic.FunctorExpression(
            functor=self._parse_rec(expression["functor"]),
            arguments=[self._parse_rec(a) for a in expression["arguments"]],
        )

    def parse_predicate_expression(self, expression: dict) -> logic.PredicateExpression:
        return logic.PredicateExpression(
            predicate=self._parse_rec(expression["functor"]),
            arguments=[self._parse_rec(a) for a in expression["arguments"]],
        )

    def parse_conditional(self, conditional: dict) -> logic.Conditional:
        return logic.Conditional(
            if_clause=self._parse_rec(conditional["if_clause"]),
            then_clause=self._parse_rec(conditional["then_clause"]),
            else_clause=self._parse_rec(conditional["else_clause"]),
        )

    def parse_import(self, imp: dict) -> logic.Import:
        return logic.Import(path=imp["path"])

    def parse_variable(self, variable: dict) -> logic.Variable:
        return logic.Variable(symbol=variable["symbol"])

    def parse_constant(self, variable: dict):
        return logic.Constant(symbol=variable["symbol"])

    def parse_problem(self, problem: dict) -> logic.Problem:
        return logic.Problem(
            premises=[self._parse_rec(p) for p in problem["premises"]],
            conjecture=self._parse_rec(problem["conjecture"]),
        )



import unittest

from gavel.dialects.tptp.antlr4 import tptp_v7_0_0_0Parser
from gavel.dialects.tptp.parser import TPTPParser


class TestFOL(unittest.TestCase):
    def test_symbols(self):
        processor = TPTPParser()
        result = list(
            processor.load_expressions_from_file("tests/files/single_line_fof.txt")
        )
        self.assertEqual(len(result), 1, "Single line was not parsed as single line")
        line = result[0]
        self.assertSetEqual(set(line.symbols()), {"ismeet", "leq"})



import os
import pickle as pkl
import re
import sys
from typing import Iterable

import requests
from antlr4 import CommonTokenStream
from antlr4 import InputStream

import gavel.config.settings as settings
import gavel.dialects.db.structures as db
from gavel.config import settings as settings
from gavel.dialects.base.parser import LogicParser
from gavel.dialects.base.parser import ParserException
from gavel.dialects.base.parser import ProblemParser
from gavel.dialects.base.parser import ProofParser
from gavel.dialects.db.compiler import DBCompiler
from gavel.dialects.db.connection import get_or_create
from gavel.dialects.db.connection import get_or_None
from gavel.dialects.db.connection import with_session
from gavel.dialects.tptp.sources import InferenceSource
from gavel.dialects.tptp.sources import Input
from gavel.dialects.tptp.sources import InternalSource
from gavel.logic import logic
from gavel.logic import problem
from gavel.logic.logic import LogicElement
from gavel.logic.problem import AnnotatedFormula
from gavel.logic.problem import FormulaRole
from gavel.logic.problem import Problem
from gavel.logic.proof import Axiom
from gavel.logic.proof import Inference
from gavel.logic.proof import Introduction
from gavel.logic.proof import LinearProof
from gavel.logic.proof import ProofStep

from .antlr4.flattening import FOFFlatteningVisitor
from .antlr4.tptp_v7_0_0_0Lexer import tptp_v7_0_0_0Lexer
from .antlr4.tptp_v7_0_0_0Parser import tptp_v7_0_0_0Parser

sys.setrecursionlimit(10000)

form_expression = re.compile(r"^(?!%|\n)(?P<logic>[^(]*)\([^.]*\)\.\S*$")


class TPTPParser(LogicParser):
    visitor = FOFFlatteningVisitor()

    def folder_processor(self, path, file_processor, *args, **kwargs):
        for root, dirs, files in os.walk(path):
            for file in files:
                if not file == "README":
                    f = os.path.join(root, file)
                    yield file_processor(f, *args, **kwargs)

    def problemset_processor(self, *args, **kwargs):
        tptp_root = kwargs.get("tptp_root", settings.TPTP_ROOT)
        return self.folder_processor(
            os.path.join(tptp_root, "Problems"), self.problem_processor
        )

    def axiomsets_processor(self, *args, **kwargs):
        tptp_root = kwargs.get("tptp_root", settings.TPTP_ROOT)
        return self.folder_processor(
            os.path.join(tptp_root, "Axioms"), self.axiomset_processor
        )

    def axiomset_processor(self, path, *args, **kwargs):
        for item in self.load_expressions_from_file(path):
            yield self.formula_processor(item, *args, **kwargs)

    def formula_processor(self, formula, *args, orig=None, **kwargs):
        return formula

    def stream_formula_lines(self, lines: Iterable[str], **kwargs):
        buffer = ""
        for line in lines:
            if not line.startswith("%") and not line.startswith("\n"):
                buffer += line.strip()
                if buffer.endswith("."):
                    yield buffer
                    buffer = ""
        if buffer:
            raise Exception('Unprocessed input: """%s"""' % buffer)

    def load_single_from_string(self, string: str, *args, **kwargs):
        s = tptp_v7_0_0_0Parser(
            CommonTokenStream(tptp_v7_0_0_0Lexer(InputStream(string)))
        )
        return s.tptp_input()

    def load_expressions_from_file(
        self, path, *args, **kwargs
    ) -> Iterable[LogicElement]:
        with open(path) as infile:
            lines = infile.readlines()
            for line in self.load_many(lines):
                yield self.parse(line)

    def load_many(
        self, lines: Iterable[str], *args, **kwargs
    ) -> Iterable[LogicElement]:
        return map(
            self.load_single_from_string, self.stream_formula_lines(lines, **kwargs)
        )

    def parse(self, tree, *args, **kwargs):
        return self.visitor.visit(tree)

    def problem_processor(self, path, *args, load_imports=False, **kwargs):
        axioms = []
        imports = []
        conjectures = []
        for raw_line in self.load_expressions_from_file(path, *args, **kwargs):
            line = self.formula_processor(raw_line)
            if isinstance(line, logic.Import):
                imports.append(line)
            elif isinstance(line, AnnotatedFormula):
                if line.role in (
                    FormulaRole.CONJECTURE,
                    FormulaRole.NEGATED_CONJECTURE,
                ):
                    conjectures.append(line)
                else:
                    axioms.append(line)

        for im in imports:
            for imported_axiom in self.load_expressions_from_file(
                os.path.join(settings.TPTP_ROOT, im.path), *args, **kwargs
            ):
                axioms.append(imported_axiom)

        for conjecture in conjectures:
            yield Problem(premises=axioms, imports=imports, conjecture=conjecture)


class StorageProcessor(TPTPParser):
    def __init__(self):
        self.compiler = DBCompiler()

    @with_session
    def problem_processor(self, path, *args, **kwargs):
        session = kwargs.get("session")
        source, s_created = get_or_create(session, db.Source, path=path)
        # problem = get_or_None(session, db.Problem, source=source.id)
        problems = []
        premises = []
        conjectures = []
        if s_created:
            for line in self.load_expressions_from_file(path):
                if isinstance(line, logic.Import):
                    imported_source = (
                        session.query(db.Source).filter_by(path=line.path).first()
                    )
                    if imported_source:
                        axiomset = (
                            session.query(db.Formula)
                            .filter_by(source=imported_source)
                            .all()
                        )
                    else:
                        axiomset = self.axiomset_processor(
                            os.path.join(settings.TPTP_ROOT, line.path),
                            session=session,
                            commit=False,
                        )
                        # raise Exception("Source (%s) not found" % line.path)
                    for axiom in axiomset:
                        premises.append(axiom)
                elif isinstance(line, AnnotatedFormula):
                    if line.role in (
                        problem.FormulaRole.CONJECTURE,
                        problem.FormulaRole.NEGATED_CONJECTURE,
                    ):
                        conjecture = self.formula_processor(
                            line, *args, source=source, force_creation=True, **kwargs
                        )
                        session.add(conjecture)
                        conjectures.append(conjecture)
                    else:
                        premise = self.formula_processor(
                            line, *args, source=source, force_creation=True, **kwargs
                        )
                        session.add(premise)
                        premises.append(premise)
                else:
                    raise NotImplementedError
            for conjecture in conjectures:
                p = db.Problem(premises=premises, source=source, conjecture=conjecture)
                session.add(p)
                problems.append(p)
            session.commit()
            return problems
        else:
            return session.query(db.Problem).filter_by(source=source).all()

    def formula_processor(
        self, formula: AnnotatedFormula, *args, force_creation=False, **kwargs
    ):
        source = kwargs.get("source")
        session = kwargs.get("session")
        if force_creation or source.id is None:
            formula_obj = db.Formula(
                name=formula.name, source=source, json=self.compiler.visit(formula.formula)
            )
            session.add(formula_obj)
        else:
            formula_obj = get_or_None(
                session, db.Formula, name=formula.name, source=source
            )
            if formula_obj is None:
                raise Exception
        return formula_obj

    @with_session
    def axiomset_processor(self, path, *args, **kwargs):
        session = kwargs.get("session")
        source, created = get_or_create(session, db.Source, path=path)
        commit = kwargs.get("commit", False)
        if created:
            result = [
                self.formula_processor(formula=item, source=source, *args, **kwargs)
                for item in self.load_expressions_from_file(path)
            ]
            if commit:
                session.commit()
            return result
        else:
            return session.query(db.Formula).filter_by(source=source).all()

def all_axioms(processor):
    files = [
        "Axioms/HWC001-0.ax",
        "Axioms/SET001-2.ax",
        "Axioms/LAT001-4.ax",
        "Axioms/GRP003-0.ax",
        "Axioms/SET004-1.ax",
        "Axioms/COM001+1.ax",
        "Axioms/CAT003-0.ax",
        "Axioms/SYN000-0.ax",
        "Axioms/LCL007+5.ax",
        "Axioms/CSR003+3.ax",
        "Axioms/KLE001+1.ax",
        "Axioms/GEO006+5.ax",
        "Axioms/MED001+1.ax",
        "Axioms/GEO004+2.ax",
        "Axioms/HAL001+0.ax",
        "Axioms/SWV009+0.ax",
        "Axioms/PHI001^0.ax",
        "Axioms/LCL013^0.ax",
        "Axioms/LCL013^2.ax",
        "Axioms/GEO008+0.ax",
        "Axioms/SWV006-3.ax",
        "Axioms/CSR002+3.ax",
        "Axioms/GEO005-0.ax",
        "Axioms/HWV002-2.ax",
        "Axioms/HWV002-1.ax",
        "Axioms/HWV001-2.ax",
        "Axioms/LCL006+4.ax",
        "Axioms/LCL008^0.ax",
        "Axioms/KLE001+4.ax",
        # BUG: , 'Axioms/KRS001+0.ax',
        "Axioms/LCL007+1.ax",
        "Axioms/GEO001-1.ax",
        "Axioms/SWV005-1.ax",
        "Axioms/ALG002+0.ax",
        "Axioms/DAT004=0.ax",
        "Axioms/GRP008-0.ax",
        "Axioms/PUZ004-0.ax",
        "Axioms/GRP004-1.ax",
        "Axioms/RNG002-0.ax",
        "Axioms/SET008^2.ax",
        "Axioms/LDA001-0.ax",
        "Axioms/CSR001+0.ax",
        "Axioms/GEO006+0.ax",
        "Axioms/LCL012^0.ax",
        "Axioms/CAT002-0.ax",
        "Axioms/LAT002-0.ax",
        "Axioms/LAT006-1.ax",
        "Axioms/LCL009^0.ax",
        "Axioms/SET001-0.ax",
        "Axioms/SWV003+0.ax",
        "Axioms/HEN001-0.ax",
        "Axioms/SET006+4.ax",
        "Axioms/GRP008-1.ax",
        "Axioms/NUM001-2.ax",
        "Axioms/GEO006+3.ax",
        "Axioms/HWV001-1.ax",
        "Axioms/GEO004+3.ax",
        # "Axioms/MAT001^0.ax",
        "Axioms/MSC001-2.ax",
        "Axioms/GRP004-2.ax",
        "Axioms/RNG003-0.ax",
        "Axioms/HEN002-0.ax",
        # BUG: , 'Axioms/BIO001+0.ax',
        "Axioms/SET009^0.ax",
        "Axioms/HWV002-0.ax",
        "Axioms/SET008^0.ax",
        "Axioms/LCL006+2.ax",
        "Axioms/GEO010+1.ax",
        "Axioms/LAT001-0.ax",
        "Axioms/LCL001-2.ax",
        "Axioms/GEO006+6.ax",
        "Axioms/GEO002-0.ax",
        "Axioms/SWV007+4.ax",
        "Axioms/GEO006+2.ax",
        "Axioms/MGT001-0.ax",
        "Axioms/PUZ006+0.ax",
        "Axioms/SWV007+2.ax",
        "Axioms/SWB001+0.ax",
        "Axioms/LCL016^0.ax",
        "Axioms/SET004-0.ax",
        "Axioms/GEO004-2.ax",
        "Axioms/TOP001-0.ax",
        "Axioms/ANA001-0.ax",
        "Axioms/PLA001-1.ax",
        "Axioms/LCL004-1.ax",
        "Axioms/SWB002+0.ax",
        "Axioms/LCL007+4.ax",
        "Axioms/SWV004-0.ax",
        "Axioms/GEO003-0.ax",
        "Axioms/LCL013^5.ax",
        "Axioms/AGT001+1.ax",
        "Axioms/KLE001+7.ax",
        "Axioms/GRA001+0.ax",
        "Axioms/AGT001+0.ax",
        "Axioms/SET006+2.ax",
        "Axioms/KLE001+5.ax",
        "Axioms/GRP001-0.ax",
        "Axioms/MSC001-1.ax",
        "Axioms/PRD001+0.ax",
        "Axioms/LCL006+0.ax",
        "Axioms/DAT002=0.ax",
        "Axioms/SWV008^0.ax",
        "Axioms/SWV001-0.ax",
        "Axioms/SET006+1.ax",
        "Axioms/GEO002-2.ax",
        "Axioms/LCL007+3.ax",
        "Axioms/SWC001-0.ax",
        "Axioms/SET001-1.ax",
        "Axioms/SWV005-0.ax",
        "Axioms/LCL013^4.ax",
        "Axioms/KLE003+0.ax",
        "Axioms/FLD001-0.ax",
        "Axioms/GEO004-0.ax",
        "Axioms/KLE001+0.ax",
        "Axioms/SWV005-7.ax",
        "Axioms/NUM005+1.ax",
        "Axioms/GRP003+0.ax",
        "Axioms/GRP004-0.ax",
        "Axioms/COL002-0.ax",
        "Axioms/GEO002-3.ax",
        "Axioms/AGT001+2.ax",
        "Axioms/PUZ005+0.ax",
        "Axioms/SET006+0.ax",
        "Axioms/NUM005+0.ax",
        "Axioms/LCL013^6.ax",
        "Axioms/KLE004+0.ax",
        "Axioms/MGT001+0.ax",
        "Axioms/SWV007+1.ax",
        "Axioms/LCL014^0.ax",
        "Axioms/GRP003-1.ax",
        "Axioms/QUA001^1.ax",
        "Axioms/GEO004-3.ax",
        "Axioms/NUM001-1.ax",
        "Axioms/GRP004+0.ax",
        "Axioms/BOO002-0.ax",
        "Axioms/LAT001-1.ax",
        "Axioms/COM001+0.ax",
        "Axioms/GEO006+4.ax",
        "Axioms/SYN000_0.ax",
        "Axioms/ROB001-1.ax",
        "Axioms/LCL007+0.ax",
        "Axioms/GEO002-1.ax",
        "Axioms/LCL007+6.ax",
        "Axioms/SWV013-0.ax",
        "Axioms/SYN002+0.ax",
        "Axioms/NUM001-0.ax",
        "Axioms/BOO003-0.ax",
        "Axioms/SWV005-5.ax",
        "Axioms/REL001+0.ax",
        "Axioms/LCL007+2.ax",
        "Axioms/LAT004-0.ax",
        "Axioms/SWV005-6.ax",
        "Axioms/SET003-0.ax",
        "Axioms/HWV001-0.ax",
        "Axioms/RNG005-0.ax",
        # BUG:, 'Axioms/KRS001+1.ax',
        "Axioms/LCL001-1.ax",
        "Axioms/LAT001-2.ax",
        "Axioms/LCL003-0.ax",
        "Axioms/SWV002-0.ax",
        "Axioms/KLE001+2.ax",
        "Axioms/SWB003+1.ax",
        "Axioms/SWV005-2.ax",
        "Axioms/GEO004-1.ax",
        "Axioms/SWV007+3.ax",
        "Axioms/GEO004+0.ax",
        "Axioms/COL001-0.ax",
        "Axioms/KLE002+0.ax",
        "Axioms/KLE001+6.ax",
        "Axioms/GEO011+0.ax",
        "Axioms/SWV010^0.ax",
        "Axioms/SWV006-2.ax",
        "Axioms/CAT004-0.ax",
        "Axioms/GRP007+0.ax",
        "Axioms/GEO006+1.ax",
        "Axioms/NUM004-0.ax",
        "Axioms/GEO007+1.ax",
        "Axioms/SWV008^2.ax",
        "Axioms/LCL004-2.ax",
        "Axioms/SET008^1.ax",
        "Axioms/DAT006=0.ax",
        "Axioms/BOO004-0.ax",
        "Axioms/QUA001^0.ax",
        "Axioms/LCL016^1.ax",
        "Axioms/GRP005-0.ax",
        "Axioms/PLA001-0.ax",
        "Axioms/SWV005-4.ax",
        "Axioms/LCL004-0.ax",
        "Axioms/NUM003-0.ax",
        "Axioms/SET001-3.ax",
        "Axioms/KLE001+3.ax",
        "Axioms/SWV008^1.ax",
        "Axioms/DAT003=0.ax",
        "Axioms/SET006+3.ax",
        "Axioms/LCL006+1.ax",
        "Axioms/NUM005+2.ax",
        "Axioms/LCL010^0.ax",
        "Axioms/NUM002-0.ax",
        "Axioms/REL001-1.ax",
        "Axioms/DAT001=0.ax",
        "Axioms/LAT005-0.ax",
        "Axioms/BOO001-0.ax",
        "Axioms/ANA003-0.ax",
        "Axioms/LCL001-0.ax",
        "Axioms/SWV012+0.ax",
        "Axioms/LCL006+3.ax",
        "Axioms/GEO007+0.ax",
        "Axioms/SWB003+0.ax",
        "Axioms/LCL017^0.ax",
        "Axioms/LCL015^0.ax",
        "Axioms/LCL015^1.ax",
        "Axioms/LAT006-0.ax",
        "Axioms/PUZ002-0.ax",
        "Axioms/ANA002-0.ax",
        "Axioms/HWV004-0.ax",
        "Axioms/REL001+1.ax",
        "Axioms/SYN000^0.ax",
        "Axioms/LAT006-2.ax",
        "Axioms/LCL013^3.ax",
        "Axioms/LAT001-3.ax",
        "Axioms/PUZ001-0.ax",
        "Axioms/SET002-0.ax",
        "Axioms/SWV006-1.ax",
        "Axioms/PLA002+0.ax",
        "Axioms/LCL002-0.ax",
        "Axioms/SWV005-3.ax",
        "Axioms/SWV006-0.ax",
        "Axioms/REL001-0.ax",
        "Axioms/DAT002=1.ax",
        "Axioms/LCL011^0.ax",
        "Axioms/PUZ005-0.ax",
        "Axioms/GEO004+1.ax",
        "Axioms/LCL006+5.ax",
        "Axioms/ALG001-0.ax",
        "Axioms/HEN003-0.ax",
        "Axioms/PUZ003-0.ax",
        "Axioms/SWV011+0.ax",
        "Axioms/GRP006-0.ax",
        "Axioms/SWC001+0.ax",
        "Axioms/MSC001-0.ax",
        "Axioms/SYN001-0.ax",
        "Axioms/RNG001-0.ax",
        "Axioms/HWV003-0.ax",
        "Axioms/SWV007+0.ax",
        "Axioms/NUM006^0.ax",
        "Axioms/HWC002-0.ax",
        "Axioms/LAT003-0.ax",
        "Axioms/FLD002-0.ax",
        "Axioms/LCL013^1.ax",
        "Axioms/GEO009+0.ax",
        "Axioms/LCL002-1.ax",
        "Axioms/COL002-1.ax",
        "Axioms/GEO001-0.ax",
        "Axioms/CAT001-0.ax",
        "Axioms/SET005+0.ax",
        "Axioms/DAT005=0.ax",
        "Axioms/ROB001-0.ax",
        "Axioms/LCL005-0.ax",
        "Axioms/RNG004-0.ax",
        "Axioms/SYN000+0.ax",
        "Axioms/GRP002-0.ax",
        "Axioms/ALG003^0.ax",
        "Axioms/GRP003-2.ax",
        "Axioms/SET007/SET007+872.ax",
        "Axioms/SET007/SET007+598.ax",
        "Axioms/SET007/SET007+871.ax",
        "Axioms/SET007/SET007+529.ax",
        "Axioms/SET007/SET007+570.ax",
        "Axioms/SET007/SET007+408.ax",
        "Axioms/SET007/SET007+190.ax",
        "Axioms/SET007/SET007+209.ax",
        "Axioms/SET007/SET007+399.ax",
        "Axioms/SET007/SET007+2.ax",
        "Axioms/SET007/SET007+922.ax",
        "Axioms/SET007/SET007+747.ax",
        "Axioms/SET007/SET007+900.ax",
        "Axioms/SET007/SET007+811.ax",
        "Axioms/SET007/SET007+640.ax",
        "Axioms/SET007/SET007+916.ax",
        "Axioms/SET007/SET007+150.ax",
        "Axioms/SET007/SET007+244.ax",
        "Axioms/SET007/SET007+665.ax",
        "Axioms/SET007/SET007+582.ax",
        "Axioms/SET007/SET007+301.ax",
        "Axioms/SET007/SET007+398.ax",
        "Axioms/SET007/SET007+414.ax",
        "Axioms/SET007/SET007+420.ax",
        "Axioms/SET007/SET007+660.ax",
        "Axioms/SET007/SET007+734.ax",
        "Axioms/SET007/SET007+839.ax",
        "Axioms/SET007/SET007+56.ax",
        "Axioms/SET007/SET007+622.ax",
        "Axioms/SET007/SET007+254.ax",
        "Axioms/SET007/SET007+170.ax",
        "Axioms/SET007/SET007+642.ax",
        "Axioms/SET007/SET007+171.ax",
        "Axioms/SET007/SET007+432.ax",
        "Axioms/SET007/SET007+499.ax",
        "Axioms/SET007/SET007+210.ax",
        "Axioms/SET007/SET007+772.ax",
        "Axioms/SET007/SET007+633.ax",
        "Axioms/SET007/SET007+720.ax",
        "Axioms/SET007/SET007+455.ax",
        "Axioms/SET007/SET007+257.ax",
        "Axioms/SET007/SET007+714.ax",
        "Axioms/SET007/SET007+919.ax",
        "Axioms/SET007/SET007+779.ax",
        "Axioms/SET007/SET007+600.ax",
        "Axioms/SET007/SET007+344.ax",
        "Axioms/SET007/SET007+796.ax",
        "Axioms/SET007/SET007+809.ax",
        "Axioms/SET007/SET007+726.ax",
        "Axioms/SET007/SET007+744.ax",
        "Axioms/SET007/SET007+39.ax",
        "Axioms/SET007/SET007+865.ax",
        "Axioms/SET007/SET007+151.ax",
        "Axioms/SET007/SET007+235.ax",
        "Axioms/SET007/SET007+220.ax",
        "Axioms/SET007/SET007+503.ax",
        "Axioms/SET007/SET007+448.ax",
        "Axioms/SET007/SET007+904.ax",
        "Axioms/SET007/SET007+158.ax",
        "Axioms/SET007/SET007+338.ax",
        "Axioms/SET007/SET007+119.ax",
        "Axioms/SET007/SET007+705.ax",
        "Axioms/SET007/SET007+206.ax",
        "Axioms/SET007/SET007+827.ax",
        "Axioms/SET007/SET007+155.ax",
        "Axioms/SET007/SET007+626.ax",
        "Axioms/SET007/SET007+601.ax",
        "Axioms/SET007/SET007+153.ax",
        "Axioms/SET007/SET007+332.ax",
        "Axioms/SET007/SET007+116.ax",
        "Axioms/SET007/SET007+195.ax",
        "Axioms/SET007/SET007+778.ax",
        "Axioms/SET007/SET007+80.ax",
        "Axioms/SET007/SET007+434.ax",
        "Axioms/SET007/SET007+790.ax",
        "Axioms/SET007/SET007+853.ax",
        "Axioms/SET007/SET007+680.ax",
        "Axioms/SET007/SET007+316.ax",
        "Axioms/SET007/SET007+488.ax",
        "Axioms/SET007/SET007+712.ax",
        "Axioms/SET007/SET007+471.ax",
        "Axioms/SET007/SET007+90.ax",
        "Axioms/SET007/SET007+561.ax",
        "Axioms/SET007/SET007+550.ax",
        "Axioms/SET007/SET007+444.ax",
        "Axioms/SET007/SET007+129.ax",
        "Axioms/SET007/SET007+616.ax",
        "Axioms/SET007/SET007+409.ax",
        "Axioms/SET007/SET007+318.ax",
        "Axioms/SET007/SET007+586.ax",
        "Axioms/SET007/SET007+852.ax",
        "Axioms/SET007/SET007+699.ax",
        "Axioms/SET007/SET007+73.ax",
        "Axioms/SET007/SET007+57.ax",
        "Axioms/SET007/SET007+429.ax",
        "Axioms/SET007/SET007+585.ax",
        "Axioms/SET007/SET007+75.ax",
        "Axioms/SET007/SET007+866.ax",
        "Axioms/SET007/SET007+53.ax",
        "Axioms/SET007/SET007+212.ax",
        "Axioms/SET007/SET007+595.ax",
        "Axioms/SET007/SET007+178.ax",
        "Axioms/SET007/SET007+330.ax",
        "Axioms/SET007/SET007+26.ax",
        "Axioms/SET007/SET007+121.ax",
        "Axioms/SET007/SET007+468.ax",
        "Axioms/SET007/SET007+801.ax",
        "Axioms/SET007/SET007+903.ax",
        "Axioms/SET007/SET007+909.ax",
        "Axioms/SET007/SET007+232.ax",
        "Axioms/SET007/SET007+840.ax",
        "Axioms/SET007/SET007+287.ax",
        "Axioms/SET007/SET007+487.ax",
        "Axioms/SET007/SET007+505.ax",
        "Axioms/SET007/SET007+157.ax",
        "Axioms/SET007/SET007+216.ax",
        "Axioms/SET007/SET007+203.ax",
        "Axioms/SET007/SET007+722.ax",
        "Axioms/SET007/SET007+844.ax",
        "Axioms/SET007/SET007+379.ax",
        "Axioms/SET007/SET007+632.ax",
        "Axioms/SET007/SET007+438.ax",
        "Axioms/SET007/SET007+538.ax",
        "Axioms/SET007/SET007+282.ax",
        "Axioms/SET007/SET007+767.ax",
        "Axioms/SET007/SET007+423.ax",
        "Axioms/SET007/SET007+855.ax",
        "Axioms/SET007/SET007+89.ax",
        "Axioms/SET007/SET007+602.ax",
        "Axioms/SET007/SET007+66.ax",
        "Axioms/SET007/SET007+743.ax",
        "Axioms/SET007/SET007+321.ax",
        "Axioms/SET007/SET007+9.ax",
        "Axioms/SET007/SET007+882.ax",
        "Axioms/SET007/SET007+797.ax",
        "Axioms/SET007/SET007+194.ax",
        "Axioms/SET007/SET007+41.ax",
        "Axioms/SET007/SET007+461.ax",
        "Axioms/SET007/SET007+521.ax",
        "Axioms/SET007/SET007+638.ax",
        "Axioms/SET007/SET007+87.ax",
        "Axioms/SET007/SET007+706.ax",
        "Axioms/SET007/SET007+3.ax",
        "Axioms/SET007/SET007+393.ax",
        "Axioms/SET007/SET007+715.ax",
        "Axioms/SET007/SET007+342.ax",
        "Axioms/SET007/SET007+908.ax",
        "Axioms/SET007/SET007+307.ax",
        "Axioms/SET007/SET007+534.ax",
        "Axioms/SET007/SET007+345.ax",
        "Axioms/SET007/SET007+428.ax",
        "Axioms/SET007/SET007+667.ax",
        "Axioms/SET007/SET007+319.ax",
        "Axioms/SET007/SET007+300.ax",
        "Axioms/SET007/SET007+708.ax",
        "Axioms/SET007/SET007+285.ax",
        "Axioms/SET007/SET007+897.ax",
        "Axioms/SET007/SET007+164.ax",
        "Axioms/SET007/SET007+808.ax",
        "Axioms/SET007/SET007+620.ax",
        "Axioms/SET007/SET007+842.ax",
        "Axioms/SET007/SET007+389.ax",
        "Axioms/SET007/SET007+163.ax",
        "Axioms/SET007/SET007+635.ax",
        "Axioms/SET007/SET007+686.ax",
        "Axioms/SET007/SET007+662.ax",
        "Axioms/SET007/SET007+519.ax",
        "Axioms/SET007/SET007+243.ax",
        "Axioms/SET007/SET007+407.ax",
        "Axioms/SET007/SET007+187.ax",
        "Axioms/SET007/SET007+693.ax",
        "Axioms/SET007/SET007+42.ax",
        "Axioms/SET007/SET007+61.ax",
        "Axioms/SET007/SET007+740.ax",
        "Axioms/SET007/SET007+813.ax",
        "Axioms/SET007/SET007+695.ax",
        "Axioms/SET007/SET007+78.ax",
        "Axioms/SET007/SET007+552.ax",
        "Axioms/SET007/SET007+732.ax",
        "Axioms/SET007/SET007+553.ax",
        "Axioms/SET007/SET007+29.ax",
        "Axioms/SET007/SET007+161.ax",
        "Axioms/SET007/SET007+22.ax",
        "Axioms/SET007/SET007+666.ax",
        "Axioms/SET007/SET007+17.ax",
        "Axioms/SET007/SET007+918.ax",
        "Axioms/SET007/SET007+533.ax",
        "Axioms/SET007/SET007+281.ax",
        "Axioms/SET007/SET007+474.ax",
        "Axioms/SET007/SET007+62.ax",
        "Axioms/SET007/SET007+498.ax",
        "Axioms/SET007/SET007+314.ax",
        "Axioms/SET007/SET007+383.ax",
        "Axioms/SET007/SET007+196.ax",
        "Axioms/SET007/SET007+826.ax",
        "Axioms/SET007/SET007+925.ax",
        "Axioms/SET007/SET007+44.ax",
        "Axioms/SET007/SET007+239.ax",
        "Axioms/SET007/SET007+386.ax",
        "Axioms/SET007/SET007+86.ax",
        "Axioms/SET007/SET007+15.ax",
        "Axioms/SET007/SET007+895.ax",
        "Axioms/SET007/SET007+275.ax",
        "Axioms/SET007/SET007+451.ax",
        "Axioms/SET007/SET007+272.ax",
        "Axioms/SET007/SET007+664.ax",
        "Axioms/SET007/SET007+435.ax",
        "Axioms/SET007/SET007+179.ax",
        "Axioms/SET007/SET007+91.ax",
        "Axioms/SET007/SET007+673.ax",
        "Axioms/SET007/SET007+416.ax",
        "Axioms/SET007/SET007+639.ax",
        "Axioms/SET007/SET007+248.ax",
        "Axioms/SET007/SET007+637.ax",
        "Axioms/SET007/SET007+340.ax",
        "Axioms/SET007/SET007+142.ax",
        "Axioms/SET007/SET007+237.ax",
        "Axioms/SET007/SET007+803.ax",
        "Axioms/SET007/SET007+392.ax",
        "Axioms/SET007/SET007+396.ax",
        "Axioms/SET007/SET007+576.ax",
        "Axioms/SET007/SET007+439.ax",
        "Axioms/SET007/SET007+88.ax",
        "Axioms/SET007/SET007+927.ax",
        "Axioms/SET007/SET007+614.ax",
        "Axioms/SET007/SET007+905.ax",
        "Axioms/SET007/SET007+286.ax",
        "Axioms/SET007/SET007+898.ax",
        "Axioms/SET007/SET007+306.ax",
        "Axioms/SET007/SET007+229.ax",
        "Axioms/SET007/SET007+615.ax",
        "Axioms/SET007/SET007+149.ax",
        "Axioms/SET007/SET007+869.ax",
        "Axioms/SET007/SET007+671.ax",
        "Axioms/SET007/SET007+376.ax",
        "Axioms/SET007/SET007+260.ax",
        "Axioms/SET007/SET007+783.ax",
        "Axioms/SET007/SET007+222.ax",
        "Axioms/SET007/SET007+502.ax",
        "Axioms/SET007/SET007+685.ax",
        "Axioms/SET007/SET007+591.ax",
        "Axioms/SET007/SET007+261.ax",
        "Axioms/SET007/SET007+823.ax",
        "Axioms/SET007/SET007+608.ax",
        "Axioms/SET007/SET007+217.ax",
        "Axioms/SET007/SET007+46.ax",
        "Axioms/SET007/SET007+490.ax",
        "Axioms/SET007/SET007+888.ax",
        "Axioms/SET007/SET007+165.ax",
        "Axioms/SET007/SET007+663.ax",
        "Axioms/SET007/SET007+843.ax",
        "Axioms/SET007/SET007+525.ax",
        "Axioms/SET007/SET007+391.ax",
        "Axioms/SET007/SET007+305.ax",
        "Axioms/SET007/SET007+760.ax",
        "Axioms/SET007/SET007+531.ax",
        "Axioms/SET007/SET007+192.ax",
        "Axioms/SET007/SET007+670.ax",
        "Axioms/SET007/SET007+122.ax",
        "Axioms/SET007/SET007+571.ax",
        "Axioms/SET007/SET007+108.ax",
        "Axioms/SET007/SET007+298.ax",
        "Axioms/SET007/SET007+780.ax",
        "Axioms/SET007/SET007+765.ax",
        "Axioms/SET007/SET007+794.ax",
        "Axioms/SET007/SET007+127.ax",
        "Axioms/SET007/SET007+259.ax",
        "Axioms/SET007/SET007+466.ax",
        "Axioms/SET007/SET007+467.ax",
        "Axioms/SET007/SET007+403.ax",
        "Axioms/SET007/SET007+34.ax",
        "Axioms/SET007/SET007+506.ax",
        "Axioms/SET007/SET007+833.ax",
        "Axioms/SET007/SET007+183.ax",
        "Axioms/SET007/SET007+518.ax",
        "Axioms/SET007/SET007+623.ax",
        "Axioms/SET007/SET007+440.ax",
        "Axioms/SET007/SET007+290.ax",
        "Axioms/SET007/SET007+219.ax",
        "Axioms/SET007/SET007+230.ax",
        "Axioms/SET007/SET007+284.ax",
        "Axioms/SET007/SET007+141.ax",
        "Axioms/SET007/SET007+443.ax",
        "Axioms/SET007/SET007+40.ax",
        "Axioms/SET007/SET007+447.ax",
        "Axioms/SET007/SET007+856.ax",
        "Axioms/SET007/SET007+711.ax",
        "Axioms/SET007/SET007+96.ax",
        "Axioms/SET007/SET007+820.ax",
        "Axioms/SET007/SET007+5.ax",
        "Axioms/SET007/SET007+609.ax",
        "Axioms/SET007/SET007+269.ax",
        "Axioms/SET007/SET007+881.ax",
        "Axioms/SET007/SET007+924.ax",
        "Axioms/SET007/SET007+208.ax",
        "Axioms/SET007/SET007+628.ax",
        "Axioms/SET007/SET007+469.ax",
        "Axioms/SET007/SET007+477.ax",
        "Axioms/SET007/SET007+810.ax",
        "Axioms/SET007/SET007+176.ax",
        "Axioms/SET007/SET007+679.ax",
        "Axioms/SET007/SET007+19.ax",
        "Axioms/SET007/SET007+746.ax",
        "Axioms/SET007/SET007+617.ax",
        "Axioms/SET007/SET007+584.ax",
        "Axioms/SET007/SET007+876.ax",
        "Axioms/SET007/SET007+567.ax",
        "Axioms/SET007/SET007+60.ax",
        "Axioms/SET007/SET007+128.ax",
        "Axioms/SET007/SET007+563.ax",
        "Axioms/SET007/SET007+603.ax",
        "Axioms/SET007/SET007+539.ax",
        "Axioms/SET007/SET007+733.ax",
        "Axioms/SET007/SET007+483.ax",
        "Axioms/SET007/SET007+647.ax",
        "Axioms/SET007/SET007+650.ax",
        "Axioms/SET007/SET007+137.ax",
        "Axioms/SET007/SET007+198.ax",
        "Axioms/SET007/SET007+914.ax",
        "Axioms/SET007/SET007+459.ax",
        "Axioms/SET007/SET007+323.ax",
        "Axioms/SET007/SET007+485.ax",
        "Axioms/SET007/SET007+412.ax",
        "Axioms/SET007/SET007+700.ax",
        "Axioms/SET007/SET007+166.ax",
        "Axioms/SET007/SET007+829.ax",
        "Axioms/SET007/SET007+643.ax",
        "Axioms/SET007/SET007+462.ax",
        "Axioms/SET007/SET007+184.ax",
        "Axioms/SET007/SET007+289.ax",
        "Axioms/SET007/SET007+154.ax",
        "Axioms/SET007/SET007+304.ax",
        "Axioms/SET007/SET007+71.ax",
        "Axioms/SET007/SET007+807.ax",
        "Axioms/SET007/SET007+917.ax",
        "Axioms/SET007/SET007+368.ax",
        "Axioms/SET007/SET007+404.ax",
        "Axioms/SET007/SET007+651.ax",
        "Axioms/SET007/SET007+430.ax",
        "Axioms/SET007/SET007+333.ax",
        "Axioms/SET007/SET007+395.ax",
        "Axioms/SET007/SET007+283.ax",
        "Axioms/SET007/SET007+605.ax",
        "Axioms/SET007/SET007+928.ax",
        "Axioms/SET007/SET007+802.ax",
        "Axioms/SET007/SET007+51.ax",
        "Axioms/SET007/SET007+341.ax",
        "Axioms/SET007/SET007+775.ax",
        "Axioms/SET007/SET007+641.ax",
        "Axioms/SET007/SET007+380.ax",
        "Axioms/SET007/SET007+189.ax",
        "Axioms/SET007/SET007+65.ax",
        "Axioms/SET007/SET007+199.ax",
        "Axioms/SET007/SET007+390.ax",
        "Axioms/SET007/SET007+104.ax",
        "Axioms/SET007/SET007+676.ax",
        "Axioms/SET007/SET007+677.ax",
        "Axioms/SET007/SET007+636.ax",
        "Axioms/SET007/SET007+753.ax",
        "Axioms/SET007/SET007+508.ax",
        "Axioms/SET007/SET007+452.ax",
        "Axioms/SET007/SET007+263.ax",
        "Axioms/SET007/SET007+241.ax",
        "Axioms/SET007/SET007+785.ax",
        "Axioms/SET007/SET007+575.ax",
        "Axioms/SET007/SET007+348.ax",
        "Axioms/SET007/SET007+371.ax",
        "Axioms/SET007/SET007+581.ax",
        "Axioms/SET007/SET007+683.ax",
        "Axioms/SET007/SET007+139.ax",
        "Axioms/SET007/SET007+792.ax",
        "Axioms/SET007/SET007+546.ax",
        "Axioms/SET007/SET007+510.ax",
        "Axioms/SET007/SET007+147.ax",
        "Axioms/SET007/SET007+523.ax",
        "Axioms/SET007/SET007+787.ax",
        "Axioms/SET007/SET007+880.ax",
        "Axioms/SET007/SET007+535.ax",
        "Axioms/SET007/SET007+352.ax",
        "Axioms/SET007/SET007+687.ax",
        "Axioms/SET007/SET007+358.ax",
        "Axioms/SET007/SET007+28.ax",
        "Axioms/SET007/SET007+117.ax",
        "Axioms/SET007/SET007+336.ax",
        "Axioms/SET007/SET007+470.ax",
        "Axioms/SET007/SET007+296.ax",
        "Axioms/SET007/SET007+701.ax",
        "Axioms/SET007/SET007+657.ax",
        "Axioms/SET007/SET007+484.ax",
        "Axioms/SET007/SET007+204.ax",
        "Axioms/SET007/SET007+293.ax",
        "Axioms/SET007/SET007+188.ax",
        "Axioms/SET007/SET007+742.ax",
        "Axioms/SET007/SET007+49.ax",
        "Axioms/SET007/SET007+520.ax",
        "Axioms/SET007/SET007+661.ax",
        "Axioms/SET007/SET007+453.ax",
        "Axioms/SET007/SET007+850.ax",
        "Axioms/SET007/SET007+72.ax",
        "Axioms/SET007/SET007+426.ax",
        "Axioms/SET007/SET007+750.ax",
        "Axioms/SET007/SET007+43.ax",
        "Axioms/SET007/SET007+704.ax",
        "Axioms/SET007/SET007+812.ax",
        "Axioms/SET007/SET007+251.ax",
        "Axioms/SET007/SET007+594.ax",
        "Axioms/SET007/SET007+771.ax",
        "Axioms/SET007/SET007+781.ax",
        "Axioms/SET007/SET007+758.ax",
        "Axioms/SET007/SET007+655.ax",
        "Axioms/SET007/SET007+312.ax",
        "Axioms/SET007/SET007+549.ax",
        "Axioms/SET007/SET007+644.ax",
        "Axioms/SET007/SET007+696.ax",
        "Axioms/SET007/SET007+247.ax",
        "Axioms/SET007/SET007+421.ax",
        "Axioms/SET007/SET007+159.ax",
        "Axioms/SET007/SET007+335.ax",
        "Axioms/SET007/SET007+7.ax",
        "Axioms/SET007/SET007+85.ax",
        "Axioms/SET007/SET007+140.ax",
        "Axioms/SET007/SET007+381.ax",
        "Axioms/SET007/SET007+362.ax",
        "Axioms/SET007/SET007+713.ax",
        "Axioms/SET007/SET007+168.ax",
        "Axioms/SET007/SET007+558.ax",
        "Axioms/SET007/SET007+889.ax",
        "Axioms/SET007/SET007+400.ax",
        "Axioms/SET007/SET007+634.ax",
        "Axioms/SET007/SET007+495.ax",
        "Axioms/SET007/SET007+773.ax",
        "Axioms/SET007/SET007+793.ax",
        "Axioms/SET007/SET007+611.ax",
        "Axioms/SET007/SET007+899.ax",
        "Axioms/SET007/SET007+377.ax",
        "Axioms/SET007/SET007+310.ax",
        "Axioms/SET007/SET007+863.ax",
        "Axioms/SET007/SET007+883.ax",
        "Axioms/SET007/SET007+537.ax",
        "Axioms/SET007/SET007+82.ax",
        "Axioms/SET007/SET007+32.ax",
        "Axioms/SET007/SET007+543.ax",
        "Axioms/SET007/SET007+375.ax",
        "Axioms/SET007/SET007+68.ax",
        "Axioms/SET007/SET007+668.ax",
        "Axioms/SET007/SET007+213.ax",
        "Axioms/SET007/SET007+621.ax",
        "Axioms/SET007/SET007+148.ax",
        "Axioms/SET007/SET007+901.ax",
        "Axioms/SET007/SET007+294.ax",
        "Axioms/SET007/SET007+349.ax",
        "Axioms/SET007/SET007+800.ax",
        "Axioms/SET007/SET007+112.ax",
        "Axioms/SET007/SET007+832.ax",
        "Axioms/SET007/SET007+631.ax",
        "Axioms/SET007/SET007+249.ax",
        "Axioms/SET007/SET007+387.ax",
        "Axioms/SET007/SET007+245.ax",
        "Axioms/SET007/SET007+325.ax",
        "Axioms/SET007/SET007+92.ax",
        "Axioms/SET007/SET007+873.ax",
        "Axioms/SET007/SET007+926.ax",
        "Axioms/SET007/SET007+224.ax",
        "Axioms/SET007/SET007+912.ax",
        "Axioms/SET007/SET007+417.ax",
        "Axioms/SET007/SET007+337.ax",
        "Axioms/SET007/SET007+770.ax",
        "Axioms/SET007/SET007+821.ax",
        "Axioms/SET007/SET007+795.ax",
        "Axioms/SET007/SET007+324.ax",
        "Axioms/SET007/SET007+530.ax",
        "Axioms/SET007/SET007+835.ax",
        "Axioms/SET007/SET007+831.ax",
        "Axioms/SET007/SET007+846.ax",
        "Axioms/SET007/SET007+463.ax",
        "Axioms/SET007/SET007+458.ax",
        "Axioms/SET007/SET007+569.ax",
        "Axioms/SET007/SET007+460.ax",
        "Axioms/SET007/SET007+374.ax",
        "Axioms/SET007/SET007+134.ax",
        "Axioms/SET007/SET007+55.ax",
        "Axioms/SET007/SET007+449.ax",
        "Axioms/SET007/SET007+491.ax",
        "Axioms/SET007/SET007+589.ax",
        "Axioms/SET007/SET007+227.ax",
        "Axioms/SET007/SET007+95.ax",
        "Axioms/SET007/SET007+674.ax",
        "Axioms/SET007/SET007+817.ax",
        "Axioms/SET007/SET007+536.ax",
        "Axioms/SET007/SET007+911.ax",
        "Axioms/SET007/SET007+437.ax",
        "Axioms/SET007/SET007+649.ax",
        "Axioms/SET007/SET007+100.ax",
        "Axioms/SET007/SET007+551.ax",
        "Axioms/SET007/SET007+278.ax",
        "Axioms/SET007/SET007+526.ax",
        "Axioms/SET007/SET007+143.ax",
        "Axioms/SET007/SET007+522.ax",
        "Axioms/SET007/SET007+442.ax",
        "Axioms/SET007/SET007+132.ax",
        "Axioms/SET007/SET007+101.ax",
        "Axioms/SET007/SET007+654.ax",
        "Axioms/SET007/SET007+501.ax",
        "Axioms/SET007/SET007+185.ax",
        "Axioms/SET007/SET007+828.ax",
        "Axioms/SET007/SET007+532.ax",
        "Axioms/SET007/SET007+107.ax",
        "Axioms/SET007/SET007+120.ax",
        "Axioms/SET007/SET007+877.ax",
        "Axioms/SET007/SET007+557.ax",
        "Axioms/SET007/SET007+115.ax",
        "Axioms/SET007/SET007+527.ax",
        "Axioms/SET007/SET007+8.ax",
        "Axioms/SET007/SET007+624.ax",
        "Axioms/SET007/SET007+825.ax",
        "Axioms/SET007/SET007+878.ax",
        "Axioms/SET007/SET007+327.ax",
        "Axioms/SET007/SET007+11.ax",
        "Axioms/SET007/SET007+703.ax",
        "Axioms/SET007/SET007+496.ax",
        "Axioms/SET007/SET007+292.ax",
        "Axioms/SET007/SET007+497.ax",
        "Axioms/SET007/SET007+819.ax",
        "Axioms/SET007/SET007+858.ax",
        "Axioms/SET007/SET007+824.ax",
        "Axioms/SET007/SET007+277.ax",
        "Axioms/SET007/SET007+320.ax",
        "Axioms/SET007/SET007+606.ax",
        "Axioms/SET007/SET007+646.ax",
        "Axioms/SET007/SET007+566.ax",
        "Axioms/SET007/SET007+436.ax",
        "Axioms/SET007/SET007+353.ax",
        "Axioms/SET007/SET007+789.ax",
        "Axioms/SET007/SET007+59.ax",
        "Axioms/SET007/SET007+920.ax",
        "Axioms/SET007/SET007+299.ax",
        "Axioms/SET007/SET007+125.ax",
        "Axioms/SET007/SET007+406.ax",
        "Axioms/SET007/SET007+109.ax",
        "Axioms/SET007/SET007+256.ax",
        "Axioms/SET007/SET007+723.ax",
        "Axioms/SET007/SET007+450.ax",
        "Axioms/SET007/SET007+6.ax",
        "Axioms/SET007/SET007+77.ax",
        "Axioms/SET007/SET007+588.ax",
        "Axioms/SET007/SET007+252.ax",
        "Axioms/SET007/SET007+355.ax",
        "Axioms/SET007/SET007+14.ax",
        "Axioms/SET007/SET007+465.ax",
        "Axioms/SET007/SET007+697.ax",
        "Axioms/SET007/SET007+748.ax",
        "Axioms/SET007/SET007+784.ax",
        "Axioms/SET007/SET007+419.ax",
        "Axioms/SET007/SET007+30.ax",
        "Axioms/SET007/SET007+494.ax",
        "Axioms/SET007/SET007+763.ax",
        "Axioms/SET007/SET007+736.ax",
        "Axioms/SET007/SET007+741.ax",
        "Axioms/SET007/SET007+544.ax",
        "Axioms/SET007/SET007+270.ax",
        "Axioms/SET007/SET007+311.ax",
        "Axioms/SET007/SET007+69.ax",
        "Axioms/SET007/SET007+99.ax",
        "Axioms/SET007/SET007+366.ax",
        "Axioms/SET007/SET007+730.ax",
        "Axioms/SET007/SET007+253.ax",
        "Axioms/SET007/SET007+33.ax",
        "Axioms/SET007/SET007+757.ax",
        "Axioms/SET007/SET007+317.ax",
        "Axioms/SET007/SET007+18.ax",
        "Axioms/SET007/SET007+446.ax",
        "Axioms/SET007/SET007+394.ax",
        "Axioms/SET007/SET007+921.ax",
        "Axioms/SET007/SET007+884.ax",
        "Axioms/SET007/SET007+592.ax",
        "Axioms/SET007/SET007+702.ax",
        "Axioms/SET007/SET007+572.ax",
        "Axioms/SET007/SET007+98.ax",
        "Axioms/SET007/SET007+231.ax",
        "Axioms/SET007/SET007+480.ax",
        "Axioms/SET007/SET007+79.ax",
        "Axioms/SET007/SET007+23.ax",
        "Axioms/SET007/SET007+554.ax",
        "Axioms/SET007/SET007+818.ax",
        "Axioms/SET007/SET007+645.ax",
        "Axioms/SET007/SET007+754.ax",
        "Axioms/SET007/SET007+25.ax",
        "Axioms/SET007/SET007+359.ax",
        "Axioms/SET007/SET007+682.ax",
        "Axioms/SET007/SET007+885.ax",
        "Axioms/SET007/SET007+58.ax",
        "Axioms/SET007/SET007+175.ax",
        "Axioms/SET007/SET007+50.ax",
        "Axioms/SET007/SET007+724.ax",
        "Axioms/SET007/SET007+144.ax",
        "Axioms/SET007/SET007+516.ax",
        "Axioms/SET007/SET007+890.ax",
        "Axioms/SET007/SET007+745.ax",
        "Axioms/SET007/SET007+692.ax",
        "Axioms/SET007/SET007+492.ax",
        "Axioms/SET007/SET007+593.ax",
        "Axioms/SET007/SET007+776.ax",
        "Axioms/SET007/SET007+769.ax",
        "Axioms/SET007/SET007+454.ax",
        "Axioms/SET007/SET007+456.ax",
        "Axioms/SET007/SET007+20.ax",
        "Axioms/SET007/SET007+297.ax",
        "Axioms/SET007/SET007+433.ax",
        "Axioms/SET007/SET007+562.ax",
        "Axioms/SET007/SET007+509.ax",
        "Axioms/SET007/SET007+548.ax",
        "Axioms/SET007/SET007+799.ax",
        "Axioms/SET007/SET007+76.ax",
        "Axioms/SET007/SET007+725.ax",
        "Axioms/SET007/SET007+512.ax",
        "Axioms/SET007/SET007+816.ax",
        "Axioms/SET007/SET007+851.ax",
        "Axioms/SET007/SET007+879.ax",
        "Axioms/SET007/SET007+597.ax",
        "Axioms/SET007/SET007+618.ax",
        "Axioms/SET007/SET007+350.ax",
        "Axioms/SET007/SET007+672.ax",
        "Axioms/SET007/SET007+405.ax",
        "Axioms/SET007/SET007+574.ax",
        "Axioms/SET007/SET007+729.ax",
        "Axioms/SET007/SET007+211.ax",
        "Axioms/SET007/SET007+303.ax",
        "Axioms/SET007/SET007+814.ax",
        "Axioms/SET007/SET007+113.ax",
        "Axioms/SET007/SET007+200.ax",
        "Axioms/SET007/SET007+215.ax",
        "Axioms/SET007/SET007+891.ax",
        "Axioms/SET007/SET007+102.ax",
        "Axioms/SET007/SET007+160.ax",
        "Axioms/SET007/SET007+759.ax",
        "Axioms/SET007/SET007+360.ax",
        "Axioms/SET007/SET007+511.ax",
        "Axioms/SET007/SET007+242.ax",
        "Axioms/SET007/SET007+370.ax",
        "Axioms/SET007/SET007+372.ax",
        "Axioms/SET007/SET007+309.ax",
        "Axioms/SET007/SET007+441.ax",
        "Axioms/SET007/SET007+737.ax",
        "Axioms/SET007/SET007+36.ax",
        "Axioms/SET007/SET007+806.ax",
        "Axioms/SET007/SET007+727.ax",
        "Axioms/SET007/SET007+815.ax",
        "Axioms/SET007/SET007+857.ax",
        "Axioms/SET007/SET007+274.ax",
        "Axioms/SET007/SET007+97.ax",
        "Axioms/SET007/SET007+607.ax",
        "Axioms/SET007/SET007+367.ax",
        "Axioms/SET007/SET007+54.ax",
        "Axioms/SET007/SET007+38.ax",
        "Axioms/SET007/SET007+205.ax",
        "Axioms/SET007/SET007+564.ax",
        "Axioms/SET007/SET007+415.ax",
        "Axioms/SET007/SET007+236.ax",
        "Axioms/SET007/SET007+93.ax",
        "Axioms/SET007/SET007+315.ax",
        "Axioms/SET007/SET007+131.ax",
        "Axioms/SET007/SET007+373.ax",
        "Axioms/SET007/SET007+401.ax",
        "Axioms/SET007/SET007+410.ax",
        "Axioms/SET007/SET007+731.ax",
        "Axioms/SET007/SET007+418.ax",
        "Axioms/SET007/SET007+361.ax",
        "Axioms/SET007/SET007+555.ax",
        "Axioms/SET007/SET007+265.ax",
        "Axioms/SET007/SET007+596.ax",
        "Axioms/SET007/SET007+798.ax",
        "Axioms/SET007/SET007+804.ax",
        "Axioms/SET007/SET007+130.ax",
        "Axioms/SET007/SET007+678.ax",
        "Axioms/SET007/SET007+258.ax",
        "Axioms/SET007/SET007+52.ax",
        "Axioms/SET007/SET007+830.ax",
        "Axioms/SET007/SET007+238.ax",
        "Axioms/SET007/SET007+67.ax",
        "Axioms/SET007/SET007+424.ax",
        "Axioms/SET007/SET007+749.ax",
        "Axioms/SET007/SET007+782.ax",
        "Axioms/SET007/SET007+126.ax",
        "Axioms/SET007/SET007+652.ax",
        "Axioms/SET007/SET007+481.ax",
        "Axioms/SET007/SET007+191.ax",
        "Axioms/SET007/SET007+489.ax",
        "Axioms/SET007/SET007+822.ax",
        "Axioms/SET007/SET007+264.ax",
        "Axioms/SET007/SET007+849.ax",
        "Axioms/SET007/SET007+464.ax",
        "Axioms/SET007/SET007+579.ax",
        "Axioms/SET007/SET007+868.ax",
        "Axioms/SET007/SET007+834.ax",
        "Axioms/SET007/SET007+425.ax",
        "Axioms/SET007/SET007+540.ax",
        "Axioms/SET007/SET007+860.ax",
        "Axioms/SET007/SET007+276.ax",
        "Axioms/SET007/SET007+848.ax",
        "Axioms/SET007/SET007+1.ax",
        "Axioms/SET007/SET007+228.ax",
        "Axioms/SET007/SET007+625.ax",
        "Axioms/SET007/SET007+246.ax",
        "Axioms/SET007/SET007+659.ax",
        "Axioms/SET007/SET007+21.ax",
        "Axioms/SET007/SET007+10.ax",
        "Axioms/SET007/SET007+357.ax",
        "Axioms/SET007/SET007+202.ax",
        "Axioms/SET007/SET007+658.ax",
        "Axioms/SET007/SET007+755.ax",
        "Axioms/SET007/SET007+193.ax",
        "Axioms/SET007/SET007+103.ax",
        "Axioms/SET007/SET007+162.ax",
        "Axioms/SET007/SET007+735.ax",
        "Axioms/SET007/SET007+385.ax",
        "Axioms/SET007/SET007+83.ax",
        "Axioms/SET007/SET007+70.ax",
        "Axioms/SET007/SET007+124.ax",
        "Axioms/SET007/SET007+587.ax",
        "Axioms/SET007/SET007+4.ax",
        "Axioms/SET007/SET007+648.ax",
        "Axioms/SET007/SET007+63.ax",
        "Axioms/SET007/SET007+690.ax",
        "Axioms/SET007/SET007+118.ax",
        "Axioms/SET007/SET007+517.ax",
        "Axioms/SET007/SET007+322.ax",
        "Axioms/SET007/SET007+751.ax",
        "Axioms/SET007/SET007+493.ax",
        "Axioms/SET007/SET007+24.ax",
        "Axioms/SET007/SET007+268.ax",
        "Axioms/SET007/SET007+473.ax",
        "Axioms/SET007/SET007+886.ax",
        "Axioms/SET007/SET007+788.ax",
        "Axioms/SET007/SET007+180.ax",
        "Axioms/SET007/SET007+174.ax",
        "Axioms/SET007/SET007+568.ax",
        "Axioms/SET007/SET007+31.ax",
        "Axioms/SET007/SET007+719.ax",
        "Axioms/SET007/SET007+233.ax",
        "Axioms/SET007/SET007+225.ax",
        "Axioms/SET007/SET007+681.ax",
        "Axioms/SET007/SET007+838.ax",
        "Axioms/SET007/SET007+907.ax",
        "Axioms/SET007/SET007+226.ax",
        "Axioms/SET007/SET007+656.ax",
        "Axioms/SET007/SET007+547.ax",
        "Axioms/SET007/SET007+762.ax",
        "Axioms/SET007/SET007+577.ax",
        "Axioms/SET007/SET007+354.ax",
        "Axioms/SET007/SET007+887.ax",
        "Axioms/SET007/SET007+363.ax",
        "Axioms/SET007/SET007+902.ax",
        "Axioms/SET007/SET007+707.ax",
        "Axioms/SET007/SET007+214.ax",
        "Axioms/SET007/SET007+347.ax",
        "Axioms/SET007/SET007+356.ax",
        "Axioms/SET007/SET007+475.ax",
        "Axioms/SET007/SET007+786.ax",
        "Axioms/SET007/SET007+279.ax",
        "Axioms/SET007/SET007+630.ax",
        "Axioms/SET007/SET007+201.ax",
        "Axioms/SET007/SET007+81.ax",
        "Axioms/SET007/SET007+500.ax",
        "Axioms/SET007/SET007+698.ax",
        "Axioms/SET007/SET007+578.ax",
        "Axioms/SET007/SET007+47.ax",
        "Axioms/SET007/SET007+351.ax",
        "Axioms/SET007/SET007+146.ax",
        "Axioms/SET007/SET007+583.ax",
        "Axioms/SET007/SET007+504.ax",
        "Axioms/SET007/SET007+114.ax",
        "Axioms/SET007/SET007+197.ax",
        "Axioms/SET007/SET007+472.ax",
        "Axioms/SET007/SET007+413.ax",
        "Axioms/SET007/SET007+266.ax",
        "Axioms/SET007/SET007+255.ax",
        "Axioms/SET007/SET007+861.ax",
        "Axioms/SET007/SET007+382.ax",
        "Axioms/SET007/SET007+45.ax",
        "Axioms/SET007/SET007+273.ax",
        "Axioms/SET007/SET007+172.ax",
        "Axioms/SET007/SET007+560.ax",
        "Axioms/SET007/SET007+524.ax",
        "Axioms/SET007/SET007+862.ax",
        "Axioms/SET007/SET007+135.ax",
        "Axioms/SET007/SET007+326.ax",
        "Axioms/SET007/SET007+123.ax",
        "Axioms/SET007/SET007+94.ax",
        "Axioms/SET007/SET007+709.ax",
        "Axioms/SET007/SET007+106.ax",
        "Axioms/SET007/SET007+896.ax",
        "Axioms/SET007/SET007+738.ax",
        "Axioms/SET007/SET007+271.ax",
        "Axioms/SET007/SET007+145.ax",
        "Axioms/SET007/SET007+156.ax",
        "Axioms/SET007/SET007+710.ax",
        "Axioms/SET007/SET007+859.ax",
        "Axioms/SET007/SET007+694.ax",
        "Axioms/SET007/SET007+870.ax",
        "Axioms/SET007/SET007+27.ax",
        "Axioms/SET007/SET007+328.ax",
        "Axioms/SET007/SET007+841.ax",
        "Axioms/SET007/SET007+910.ax",
        "Axioms/SET007/SET007+689.ax",
        "Axioms/SET007/SET007+837.ax",
        "Axioms/SET007/SET007+111.ax",
        "Axioms/SET007/SET007+313.ax",
        "Axioms/SET007/SET007+329.ax",
        "Axioms/SET007/SET007+854.ax",
        "Axioms/SET007/SET007+378.ax",
        "Axioms/SET007/SET007+478.ax",
        "Axioms/SET007/SET007+653.ax",
        "Axioms/SET007/SET007+515.ax",
        "Axioms/SET007/SET007+48.ax",
        "Axioms/SET007/SET007+573.ax",
        "Axioms/SET007/SET007+684.ax",
        "Axioms/SET007/SET007+613.ax",
        "Axioms/SET007/SET007+590.ax",
        "Axioms/SET007/SET007+545.ax",
        "Axioms/SET007/SET007+291.ax",
        "Axioms/SET007/SET007+234.ax",
        "Axioms/SET007/SET007+331.ax",
        "Axioms/SET007/SET007+717.ax",
        "Axioms/SET007/SET007+346.ax",
        "Axioms/SET007/SET007+105.ax",
        "Axioms/SET007/SET007+791.ax",
        "Axioms/SET007/SET007+218.ax",
        "Axioms/SET007/SET007+221.ax",
        "Axioms/SET007/SET007+169.ax",
        "Axioms/SET007/SET007+110.ax",
        "Axioms/SET007/SET007+845.ax",
        "Axioms/SET007/SET007+457.ax",
        "Axioms/SET007/SET007+136.ax",
        "Axioms/SET007/SET007+295.ax",
        "Axioms/SET007/SET007+12.ax",
        "Axioms/SET007/SET007+280.ax",
        "Axioms/SET007/SET007+402.ax",
        "Axioms/SET007/SET007+427.ax",
        "Axioms/SET007/SET007+610.ax",
        "Axioms/SET007/SET007+894.ax",
        "Axioms/SET007/SET007+923.ax",
        "Axioms/SET007/SET007+223.ax",
        "Axioms/SET007/SET007+565.ax",
        "Axioms/SET007/SET007+482.ax",
        "Axioms/SET007/SET007+308.ax",
        "Axioms/SET007/SET007+186.ax",
        "Axioms/SET007/SET007+718.ax",
        "Axioms/SET007/SET007+629.ax",
        "Axioms/SET007/SET007+0.ax",
        "Axioms/SET007/SET007+181.ax",
        "Axioms/SET007/SET007+369.ax",
        "Axioms/SET007/SET007+84.ax",
        "Axioms/SET007/SET007+397.ax",
        "Axioms/SET007/SET007+182.ax",
        "Axioms/SET007/SET007+167.ax",
        "Axioms/SET007/SET007+37.ax",
        "Axioms/SET007/SET007+875.ax",
        "Axioms/SET007/SET007+542.ax",
        "Axioms/SET007/SET007+388.ax",
        "Axioms/SET007/SET007+612.ax",
        "Axioms/SET007/SET007+431.ax",
        "Axioms/SET007/SET007+339.ax",
        "Axioms/SET007/SET007+761.ax",
        "Axioms/SET007/SET007+267.ax",
        "Axioms/SET007/SET007+675.ax",
        "Axioms/SET007/SET007+867.ax",
        "Axioms/SET007/SET007+138.ax",
        "Axioms/SET007/SET007+559.ax",
        "Axioms/SET007/SET007+240.ax",
        "Axioms/SET007/SET007+514.ax",
        "Axioms/SET007/SET007+604.ax",
        "Axioms/SET007/SET007+16.ax",
        "Axioms/SET007/SET007+528.ax",
        "Axioms/SET007/SET007+764.ax",
        "Axioms/SET007/SET007+915.ax",
        "Axioms/SET007/SET007+752.ax",
        "Axioms/SET007/SET007+479.ax",
        "Axioms/SET007/SET007+384.ax",
        "Axioms/SET007/SET007+688.ax",
        "Axioms/SET007/SET007+334.ax",
        "Axioms/SET007/SET007+302.ax",
        "Axioms/SET007/SET007+691.ax",
        "Axioms/SET007/SET007+541.ax",
        "Axioms/SET007/SET007+768.ax",
        "Axioms/SET007/SET007+739.ax",
        "Axioms/SET007/SET007+177.ax",
        "Axioms/SET007/SET007+556.ax",
        "Axioms/SET007/SET007+486.ax",
        "Axioms/SET007/SET007+580.ax",
        "Axioms/SET007/SET007+445.ax",
        "Axioms/SET007/SET007+847.ax",
        "Axioms/SET007/SET007+716.ax",
        "Axioms/SET007/SET007+728.ax",
        "Axioms/SET007/SET007+207.ax",
        "Axioms/SET007/SET007+913.ax",
        "Axioms/SET007/SET007+133.ax",
        "Axioms/SET007/SET007+152.ax",
        "Axioms/SET007/SET007+756.ax",
        "Axioms/SET007/SET007+805.ax",
        "Axioms/SET007/SET007+766.ax",
        "Axioms/SET007/SET007+599.ax",
        "Axioms/SET007/SET007+411.ax",
        "Axioms/SET007/SET007+721.ax",
        "Axioms/SET007/SET007+669.ax",
        "Axioms/SET007/SET007+250.ax",
        "Axioms/SET007/SET007+892.ax",
        "Axioms/SET007/SET007+422.ax",
        "Axioms/SET007/SET007+906.ax",
        "Axioms/SET007/SET007+288.ax",
        "Axioms/SET007/SET007+774.ax",
        "Axioms/SET007/SET007+262.ax",
        "Axioms/SET007/SET007+777.ax",
        "Axioms/SET007/SET007+864.ax",
        "Axioms/SET007/SET007+619.ax",
        "Axioms/SET007/SET007+343.ax",
        "Axioms/SET007/SET007+929.ax",
        "Axioms/SET007/SET007+476.ax",
        "Axioms/SET007/SET007+513.ax",
        "Axioms/SET007/SET007+64.ax",
        "Axioms/SET007/SET007+364.ax",
        "Axioms/SET007/SET007+35.ax",
        "Axioms/SET007/SET007+893.ax",
        "Axioms/SET007/SET007+874.ax",
        "Axioms/SET007/SET007+627.ax",
        "Axioms/SET007/SET007+836.ax",
        "Axioms/SET007/SET007+13.ax",
        "Axioms/SET007/SET007+173.ax",
        "Axioms/SET007/SET007+507.ax",
        "Axioms/SET007/SET007+74.ax",
        "Axioms/SET007/SET007+365.ax",
        "Axioms/GEO010+0.ax",
        #'Axioms/CSR002+5.ax',
        # "Axioms/CSR003+2.ax",
        "Axioms/CSR001+2.ax",
        "Axioms/CSR001+3.ax",
        "Axioms/CSR004+0.ax",
        "Axioms/CSR001+1.ax",
        "Axioms/CSR003+0.ax",
        "Axioms/CSR003+5.ax",
        "Axioms/CSR003+4.ax",
        "Axioms/CSR002+0.ax",
        "Axioms/CSR002+1.ax",
        "Axioms/CSR005^0.ax",
        "Axioms/CSR002+4.ax",
        "Axioms/CSR003+1.ax",
        "Axioms/CSR002+2.ax",
        "Axioms/NLP001+0.ax",
        "Axioms/MED001+0.ax",
        "Axioms/MED002+0.ax",
    ]
    for f in files:
        print(f)
        processor.axiomset_processor(os.path.join(settings.TPTP_ROOT, f))


class TPTPProblemParser(ProblemParser):
    logic_parser_cls = TPTPParser


class SimpleTPTPProofParser(ProofParser):
    def __init__(self):
        self._tptp_parser = TPTPParser()

    def parse(self, structure: str, *args, **kwargs):
        return LinearProof(
            steps=[
                self._create_proof_step(self._tptp_parser.parse(s))
                for s in self._tptp_parser.load_many(structure.split("\n"))
            ]
        )

    def _create_proof_step(self, e: LogicElement) -> ProofStep:
        if isinstance(e, AnnotatedFormula):
            if e.role == FormulaRole.AXIOM:
                return Axiom(formula=e.formula, name=e.name)
            elif e.role == FormulaRole.PLAIN:
                if isinstance(e.annotation, InferenceSource):
                    return Inference(
                        formula=e.formula, name=e.name, antecedents=e.annotation.parents
                    )
                elif isinstance(e.annotation, InternalSource):
                    return Introduction(
                        formula=e.formula,
                        name=e.name,
                        introduction_type=e.annotation.intro_type,
                    )
            raise ParserException(e)
        else:
            raise ParserException(e)


def get_all_files(path):
    for root, dirs, files in os.walk(path):
        for file in files:
            if not file == "README":
                yield os.path.join(root, file)


def all_problems(processor):
    for f in get_all_files(os.path.join(settings.TPTP_ROOT, "Problems")):
        print(f)
        for problem in processor.problem_processor(f):
            print("done")


def all_solution(path, system=""):
    path = os.path.join(path, "Problems")
    files = get_all_files(path)
    for file in files:
        domain, problem = os.path.normpath(file).split(os.sep)[-2:]
        response = requests.get(
            "http://www.tptp.org/cgi-bin/SeeTPTP?Category=Solutions"
            "&Domain={domain}"
            "&File={problem}"
            "&System=Vampire---4.3.THM-Ref.s".format(
                domain=domain, problem=problem[:-2]
            )
        )
        print(response)


def store_problems():
    processor = StorageProcessor()
    all_problems(processor)


def store_axioms():
    processor = StorageProcessor()
    all_axioms(processor)



from typing import Iterable

from gavel.logic.proof import IntroductionType


class Annotation:
    pass


class Source:
    pass


class InternalSource(Source):
    def __init__(self, intro_type: IntroductionType, info):
        self.intro_type = intro_type
        self.info = info


class InferenceSource(Source):
    def __init__(self, rule, info, parents: Iterable[Source]):
        self.rule = rule
        self.info = info
        self.parents = parents


class FileSource(Source):
    def __init__(self, path, info):
        self.path = path
        self.info = info


class CreatorSource(Source):
    def __init__(self, creator):
        self.creator = creator


class TheorySource(Source):
    def __init__(self, theory):
        self.theory = theory


class Input(Annotation):
    pass



from itertools import chain
from typing import List

from gavel.logic.logic import LogicElement
from gavel.logic.problem import AnnotatedFormula


class Selector:
    """
    Base class for gavel selectors.
    """

    def __init__(self, premises: List[AnnotatedFormula], conjecture: AnnotatedFormula):
        self.premises = premises
        self.conjecture = conjecture

    def select(self):
        return self.premises


class Sine(Selector):
    def __init__(
        self,
        premises: List[AnnotatedFormula],
        conjecture: AnnotatedFormula,
        max_depth: int = 5,
    ):
        super(Sine, self).__init__(premises, conjecture)
        self.symbols = set(
            chain(self.conjecture.symbols(), *(p.symbols() for p in premises))
        )
        self._premise_symbols = {p: set(p.symbols()) for p in self.premises}
        self.commonness = {
            s: sum(1 for ps in self._premise_symbols.values() if s in ps)
            for s in self.symbols
        }
        self.min_commonness = min(map(lambda x: self.commonness[x], self.symbols))
        self.max_depth = max_depth

    def select(self):
        return self.calculate_triggers()

    def trigger(self, symbol, sentence: LogicElement) -> bool:
        return symbol in sentence.symbols() and all(
            self.commonness[symbol] <= self.commonness[symbol2]
            for symbol2 in sentence.symbols()
        )

    def calculate_triggers(self):
        remaining_premises = list(self.premises)
        k_triggered_symbols = set(self.conjecture.symbols())
        used_symbols = set()
        k = 0
        while k_triggered_symbols and k < self.max_depth:
            k += 1
            untriggered_premises = []
            newer_symbols = set()
            for p in remaining_premises:
                p_symbs = self._premise_symbols[p]
                # If s is k-step triggered and s triggers A, then A is k + 1-step triggered
                if any(self.trigger(s, p) for s in k_triggered_symbols):
                    newer_symbols = newer_symbols.union(p_symbs)
                    yield p
                else:
                    untriggered_premises.append(p)
            remaining_premises = untriggered_premises
            used_symbols = used_symbols.union(k_triggered_symbols)
            k_triggered_symbols = newer_symbols.difference(used_symbols)


class SineTolerance(Sine):
    def __init__(self, *args, tolerance, **kwargs):
        super(SineTolerance, self).__init__(*args, **kwargs)
        self.tolerance = tolerance

    def trigger(self, symbol, sentence: LogicElement) -> bool:
        return symbol in sentence.symbols() and all(
            self.commonness[symbol] <= self.tolerance * self.commonness[symbol2]
            for symbol2 in sentence.symbols()
        )


class SineGenerality(Sine):
    def __init__(self, *args, generality, **kwargs):
        super(SineGenerality, self).__init__(*args, **kwargs)
        self.generality = generality

    def trigger(self, symbol, sentence: LogicElement) -> bool:
        return (
            symbol in sentence.symbols()
            and self.commonness[symbol] <= self.generality
            and all(
                self.commonness[symbol] <= self.commonness[symbol2]
                for symbol2 in sentence.symbols()
            )
        )



import gavel.logic.logic as fol
from gavel.dialects.base.compiler import Compiler
from gavel.logic import problem
from gavel.dialects.db.structures import Formula

class DBCompiler(Compiler):
    def visit_quantifier(self, quantifier: fol.Quantifier):
        if quantifier.is_existential():
            return dict(type="quantifier", quantifier="existential")
        elif quantifier.is_universal():
            return dict(type="quantifier", quantifier="universial")

    def visit_formula_role(self, role: problem.FormulaRole):
        if role == problem.FormulaRole.ASSUMPTION:
            return dict(type="formula_role", formula_role="assumption")
        elif role == problem.FormulaRole.AXIOM:
            return dict(type="formula_role", formula_role="axiom")
        elif role == problem.FormulaRole.CONJECTURE:
            return dict(type="formula_role", formula_role="conjecture")
        elif role == problem.FormulaRole.COROLLARY:
            return dict(type="formula_role", formula_role="corollary")
        elif role == problem.FormulaRole.DEFINITION:
            return dict(type="formula_role", formula_role="definition")
        elif role == problem.FormulaRole.FI_DOMAIN:
            return dict(type="formula_role", formula_role="fi_domain")
        elif role == problem.FormulaRole.FI_FUNCTORS:
            return dict(type="formula_role", formula_role="fi_functors")
        elif role == problem.FormulaRole.FI_PREDICATES:
            return dict(type="formula_role", formula_role="fi_predicates")
        elif role == problem.FormulaRole.HYPOTHESIS:
            return dict(type="formula_role", formula_role="hypothesis")
        elif role == problem.FormulaRole.LEMMA:
            return dict(type="formula_role", formula_role="lemma")
        elif role == problem.FormulaRole.NEGATED_CONJECTURE:
            return dict(type="formula_role", formula_role="negated_conjecture")
        elif role == problem.FormulaRole.PLAIN:
            return dict(type="formula_role", formula_role="plain")
        elif role == problem.FormulaRole.THEOREM:
            return dict(type="formula_role", formula_role="theorem")
        elif role == problem.FormulaRole.TYPE:
            return dict(type="formula_role", formula_role="type")
        elif role == problem.FormulaRole.UNKNOWN:
            return dict(type="formula_role", formula_role="unknown")
        raise NotImplementedError

    def visit_binary_connective(self, connective: fol.BinaryConnective):
        return dict(type="binary_connective", binary_connective=connective.name.lower())

    def visit_defined_predicate(self, predicate: fol.DefinedPredicate):
        return dict(type="defined_predicate", defined_predicate=predicate.name.lower())

    def visit_unary_connective(self, connective: fol.UnaryConnective):
        return dict(type="unary_connective", unary_connective=connective.name.lower())

    def visit_unary_formula(self, formula: fol.UnaryFormula):
        return dict(
            type="unary_formula",
            formula=self.visit(formula.formula),
            connective=self.visit(formula.connective),
        )

    def visit_quantified_formula(self, formula: fol.QuantifiedFormula):
        return dict(
            type="quantified_formula",
            formula=self.visit(formula.formula),
            quantifier=self.visit(formula.quantifier),
            variables=[self.visit(v) for v in formula.variables],
        )

    def visit_annotated_formula(self, anno: problem.AnnotatedFormula, root=True):
        if root:
            return Formula(
                json=self.visit(anno.formula),
                name=self.visit(anno.name),
                logic=self.visit(anno.logic),
            )
        else:
            return dict(
                type="annotated_formula",
                formula=self.visit(anno.formula),
                name=self.visit(anno.name),
                role=self.visit(anno.role),
                logic=self.visit(anno.logic),
            )


    def visit_binary_formula(self, formula: fol.BinaryFormula):
        return dict(
            type="binary_formula",
            left=self.visit(formula.left),
            right=self.visit(formula.right),
            connective=self.visit(formula.operator),
        )

    def visit_functor_expression(self, expression: fol.FunctorExpression):
        return dict(
            type="functor_expression",
            arguments=[self.visit(a) for a in expression.arguments],
            functor=self.visit(expression.functor),
        )

    def visit_predicate_expression(self, expression: fol.PredicateExpression):
        return dict(
            type="predicate_expression",
            arguments=[self.visit(a) for a in expression.arguments],
            predicate=self.visit(expression.predicate),
        )

    def visit_conditional(self, conditional: fol.Conditional):
        return dict(
            type="conditional",
            if_clause=conditional.if_clause,
            then_clause=conditional.then_clause,
            else_clause=conditional.else_clause,
        )

    def visit_import(self, imp: fol.Import):
        return dict(type="import", path=imp.path)

    def visit_variable(self, variable: fol.Variable):
        return dict(type="variable", symbol=variable.symbol)

    def visit_constant(self, variable: fol.Variable):
        return dict(type="constant", symbol=variable.symbol)

    def visit_problem(self, problem: problem.Problem):
        return dict(
            type="problem",
            premises=[self.visit_annotated_formula(p, root=False) for p in problem.premises],
            conjecture=self.visit(problem.conjecture),
        )

    def visit_let(self, expression: fol.Let):
        return dict(
            formula=self.visit(expression.formula),
            definitions=self.visit(expression.definitions),
            types=self.visit(expression.types)
        )

    def visit_mapping_type(self, expression: fol.Subtype):
        pass

    def visit_quantified_type(self, expression: fol.QuantifiedType):
        pass

    def visit_subtype(self, expression: fol.Subtype):
        pass

    def visit_type_formula(self, formula: fol.TypeFormula):
        pass

    def visit_typed_variable(self, variable: fol.TypedVariable):
        pass



from gavel.dialects.tptp.parser import TPTPParser
from gavel.logic import logic
from gavel.logic import problem

from ..test_base.test_parser import TestLogicParser


class TestTPTPParser(TestLogicParser):
    _parser_cls = TPTPParser

    def test_verum(self):
        inp = """fof(mElmSort,axiom,(
    ! [W0] :
      ( aElement0(W0)
     => $true ) ))."""
        result = problem.AnnotatedFormula(
            logic="fof",
            name="mElmSort",
            role=problem.FormulaRole.AXIOM,
            formula=logic.QuantifiedFormula(
                quantifier=logic.Quantifier.UNIVERSAL,
                variables=[logic.Variable("W0")],
                formula=logic.BinaryFormula(
                    left=logic.FunctorExpression(
                        functor="aElement0", arguments=[logic.Variable("W0")]
                    ),
                    operator=logic.BinaryConnective.IMPLICATION,
                    right=logic.DefinedConstant.VERUM,
                ),
            ),
        )
        self.check_parser(inp, result)

class TestTHFParser(TestLogicParser):
    _parser_cls = TPTPParser

    def test_type_formula(self):
        inp = """thf(prop_a,type,(
    prop_a: $i > $o ))."""
        expected = problem.AnnotatedFormula(
            logic="thf",
            name="prop_a",
            role=problem.FormulaRole.TYPE,
            formula=logic.TypeFormula(
                name=logic.Constant("prop_a"),
                type_expression=logic.MappingType(
                    left="$i",
                    right="$o"
                )
            )
        )
        self.check_parser(inp, expected)



import gavel.dialects.tptp.sources as sources
from gavel.dialects.base.dialect import Problem
from gavel.dialects.tptp.dialect import TPTPDialect
from gavel.prover.base.interface import BaseProverInterface
from gavel.prover.base.interface import BaseResultHandler


class VampireInterface(BaseProverInterface):
    _dialect_cls = TPTPDialect

    def prove(self, problem: Problem, *args, **kwargs):
        """
        Attempt to prove a problem specified in `file`
        :param file:
        :return:
        """


class ResultHandler(BaseResultHandler):
    def get_used_axioms(self):
        raise NotImplementedError



import json
import tempfile
from typing import Iterable
from urllib.parse import quote

import requests as req

from gavel.config.settings import HETS_HOST
from gavel.config.settings import HETS_PORT
from gavel.dialects.tptp.dialect import TPTPDialect
from gavel.logic.logic import LogicElement
from gavel.logic.problem import AnnotatedFormula
from gavel.logic.problem import Problem
from gavel.logic.proof import Proof
from gavel.prover.base.interface import BaseProverInterface


class HetsCall:
    def __init__(self, paths, *args, **kwargs):
        super(HetsCall, self).__init__(*args, **kwargs)
        self.url = "http://" + HETS_HOST
        if HETS_PORT:
            self.url += ":" + str(HETS_PORT)
        self.url = "/".join([self.url] + paths)

    def get(self, path, *args, **kwargs):
        print(path)
        return self.__send(req.get, path, *args, **kwargs)

    def post(self, path, *args, data=None, **kwargs):
        return self.__send(req.post, path, *args, data=data, **kwargs)

    def __send(self, f, path, *args, **kwargs):
        return f(self.url + "/" + path, *args, **kwargs)

    @staticmethod
    def encode(path):
        return quote(path, safe="")


class HetsProve(BaseProverInterface, HetsCall):
    _prover_dialect_cls = TPTPDialect

    def __init__(self, prover_interface: BaseProverInterface, *args, **kwargs):
        super(HetsProve, self).__init__(["prove"])

    def _bootstrap_problem(self, problem: Problem):
        tf = tempfile.NamedTemporaryFile(mode="w+")
        problem_string = self.dialect.compile_problem(problem)
        tf.write(problem_string)
        tf.seek(0)
        return tf

    def _submit_problem(self, problem_instance, *args, **kwargs):
        node = str(problem_instance.name).split("/")[-1]
        iri = self.encode("file://" + str(problem_instance.name))
        response = self.post(
            iri,
            json=dict(
                format="json",
                goals=[
                    dict(
                        node=node,
                        reasonerConfiguration=dict(timeLimit=100, reasoner="Vampire"),
                    )
                ],
            ),
        )
        problem_instance.close()
        if response.status_code == 200:
            jsn = json.loads(response.content.decode("utf-8"))[0]
            assert "goals" in jsn
            goals = jsn["goals"]
            assert len(goals) == 1
            return goals[0]["prover_output"]
        else:
            raise Exception(response.content)

    def _post_process_proof(self, raw_proof_result):
        return raw_proof_result



from enum import Enum
from typing import Iterable

from gavel.logic.logic import LogicElement


class ProblemElement:
    pass


class Sentence(ProblemElement):
    def is_conjecture(self):
        raise NotImplementedError


class FormulaRole(Enum):

    __visit_name__ = "formula_role"

    AXIOM = 0
    HYPOTHESIS = 1
    DEFINITION = 2
    ASSUMPTION = 3
    LEMMA = 4
    THEOREM = 5
    COROLLARY = 6
    CONJECTURE = 7
    PLAIN = 8
    FINITE_INTERPRETATION_DOMAIN = 9
    FINITE_INTERPRETATION_FUNCTORS = 10
    FINITE_INTERPRETATION_PREDICATES = 11
    UNKNOWN = 12
    TYPE = 13
    NEGATED_CONJECTURE = 14

    def __repr__(self):
        return self.name


class AnnotatedFormula(Sentence):

    __visit_name__ = "annotated_formula"

    def __init__(
        self,
        logic,
        name: str = None,
        role: FormulaRole = None,
        formula: LogicElement = None,
        annotation=None,
    ):
        self.logic = logic
        self.name = name
        self.role = role
        self.formula = formula
        self.annotation = annotation

    def symbols(self):
        return self.formula.symbols()

    def is_conjecture(self):
        return self.role in (FormulaRole.CONJECTURE, FormulaRole.NEGATED_CONJECTURE)

    def __str__(self):
        return (
            "{logic}({name},{role},{formula})".format(
                logic=self.logic, name=self.name, role=self.role, formula=self.formula
            )
            + ("# " + str(self.annotation))
            if self.annotation
            else ""
        )

    def __eq__(self, other):
        return type(self) == type(other) and all(
            getattr(self, n) == getattr(other, n) for n in self.__dict__
        )


class Import(ProblemElement):

    __visit_name__ = "import"

    def __init__(self, path):
        self.path = path.replace("'", "")


class Problem:
    def __init__(
        self, premises: Iterable[Sentence], conjecture: Iterable[Sentence], imports=None
    ):
        self.premises = premises
        self.conjecture = conjecture
        self.imports = imports or []



from typing import Generic
from typing import Iterable
from typing import TypeVar

from gavel.logic.logic import LogicElement
from gavel.logic.problem import Problem
from gavel.logic.problem import Sentence
from gavel.logic.proof import Proof

Parseable = TypeVar("Parseable")
Target = TypeVar("Target")


class Parser(Generic[Parseable, Target]):
    def parse(self, structure: Parseable, *args, **kwargs) -> Target:
        """
        Transforms the input structure into metadata as used by the
        OpenEnergyPlatform
        Parameters
        ----------
        inp: str
            The input string that should be parsed into OEP metadata
        Returns
        -------
        OEPMetadata
            OEP metadata represented by `inp`
        """
        raise NotImplementedError

    def load_single_from_string(self, string: str, *args, **kwargs) -> Parseable:
        """
        Load a string into the structure represented by the dialect
        Parameters
        ----------
        string
        args
        kwargs
        Returns
        -------
        """
        raise NotImplementedError

    def load_many(self, string: Iterable[str]) -> Iterable[Parseable]:
        raise NotImplementedError

    def parse_single_from_string(
        self,
        string: str,
        load_args=None,
        parse_args=None,
        load_kwargs=None,
        parse_kwargs=None,
    ) -> Target:
        """
        Parse a string into OEPMetadata
        Parameters
        ----------
        string
        args
        kwargs
        Returns
        -------
        """
        return self.parse(
            self.load_single_from_string(string, *(load_args or []), **(load_kwargs or {})),
            *(parse_args or []),
            **(parse_kwargs or {})
        )

    @staticmethod
    def __unpack_file(*args, **kwargs) -> str:
        """
        Parameters
        ----------
        Returns
        -------
        """
        with open(*args, **kwargs) as inp:
            return inp.read()

    def parse_from_file(self, file_path, *args, **kwargs) -> Iterable[Target]:
        for line in self.load_many(self.__unpack_file(file_path).split("\n")):
            yield self.parse(line, *args, **kwargs)

    def is_valid(self, inp: str) -> bool:
        """
        Verify if `inp` is a sting representation that is parsable by this
        parser
        Parameters
        ----------
        inp: str
            String to verify
        Returns
        -------
        bool:
            Indicated whether this object is parsable or not
        """
        raise NotImplementedError

    def is_file_valid(self, *args, **kwargs):
        return self.is_valid(self.__unpack_file(*args, **kwargs))


class LogicParser(Parser[Parseable, LogicElement]):
    pass


class ProblemParser(Parser[Parseable, Problem]):
    logic_parser_cls = LogicParser

    def __init__(self, *args, **kwargs):
        self.logic_parser = LogicParser(*args, **kwargs)

    def parse(self, structure: Parseable, *args, **kwargs):
        premises = []
        conjectures = []
        for s in self.logic_parser.parse(structure):
            if isinstance(s, Sentence):
                if s.is_conjecture():
                    conjectures.append(s)
                else:
                    premises.append(s)
            else:
                raise ParserException("Unknown element:" + str(s))
        for c in conjectures:
            yield Problem(premises, c)


class ParserException(Exception):
    pass


class ProofParser(Parser[Parseable, Proof]):
    pass



import unittest

from gavel.dialects.tptp.compiler import TPTPCompiler
from gavel.logic import logic

from ..test_base.test_compiler import TestCompiler


class TestTPTPCompiler(TestCompiler):
    compiler = TPTPCompiler

    def test_variable(self):
        self.assert_compiler(logic.Variable("X"), "X")

    def test_connective(self):
        for input_conn, expected_conn in [
            (logic.BinaryConnective.CONJUNCTION, "&"),
            (logic.BinaryConnective.DISJUNCTION, "|"),
        ]:
            with self.subTest(input_conn=input_conn, expected_conn=expected_conn):
                self.assert_compiler(
                    logic.BinaryFormula(
                        logic.Variable("X"), input_conn, logic.Variable("Y")
                    ),
                    "X%sY" % expected_conn,
                )



from typing import Iterable

from gavel.dialects.base.compiler import Compiler
from gavel.dialects.base.parser import ProblemParser
from gavel.dialects.base.parser import ProofParser
from gavel.logic.problem import Problem
from gavel.logic.proof import Proof


class Dialect:
    _problem_parser_cls = ProblemParser
    _compiler_cls = Compiler
    _proof_parser_cls = ProofParser

    def compile(self, obj: Problem, *args, **kwargs):
        c = self._compiler_cls()
        return c.visit(obj, *args, **kwargs)

    def compile_problem(self, obj: Problem, *args, **kwargs):
        c = self._compiler_cls()
        return c.visit_problem(obj, *args, **kwargs)

    def parse_problem(self, string: str, *args, **kwargs) -> Problem:
        p = self._problem_parser_cls()
        return p.parse(string, *args, **kwargs)

    def parse_many_expressions(
        self, strings: Iterable[str], *args, **kwargs
    ) -> Iterable:
        p = self._problem_parser_cls.logic_parser_cls()
        return p.load_many(strings, *args, **kwargs)

    def parse_expression(self, string: str, *args, **kwargs) -> Problem:
        p = self._problem_parser_cls.logic_parser_cls()
        return p.parse(string, *args, **kwargs)

    def parse_expression_from_string(self, string: str, *args, **kwargs) -> Problem:
        p = self._problem_parser_cls.logic_parser_cls()
        return p.parse_single_from_string(string, *args, **kwargs)

    def compile_and_render(self, obj: Problem, *args, **kwargs):
        c = self._compiler_cls()
        compiled = c.visit(obj, *args, **kwargs)
        return r.render(compiled, *args, **kwargs)

    def parse_proof(self, prover_output) -> Proof:
        p = self._proof_parser_cls()
        return p.parse(prover_output)



"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -mgavel` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``gavel.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``gavel.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""

import click
import os

import gavel.config.settings as settings
from gavel.dialects.db.structures import store_formula, create_tables, drop_tables
import gavel.dialects.tptp.parser as build_tptp
from gavel.dialects.tptp.compiler import TPTPCompiler
from gavel.dialects.db.compiler import DBCompiler
from gavel.dialects.tptp.parser import Problem
from gavel.dialects.tptp.parser import TPTPParser
from gavel.prover.hets.interface import HetsProve
from gavel.prover.vampire.interface import VampireInterface
from gavel.selection.selector import Sine

import networkx as nx
from matplotlib import pyplot as plt
@click.group()
def db():
    pass


@click.command()
def init_db():
    """Create tables for storage of formulas"""
    create_tables()


@click.command()
@click.argument("path", default=None)
@click.option("-r", default=False)
def store(path, r):
    parser = TPTPParser()
    compiler = DBCompiler()
    for sub_path in os.listdir(path):
        sub_path = os.path.join(path, sub_path)
        if os.path.isfile(sub_path):
            print(sub_path)
            i = 0
            for formula in parser.parse_from_file(sub_path):
                i += 1
                struc = compiler.visit(formula)
                store_formula(sub_path, struc)
            print("--- %d formulas extracted ---"%i)
@click.command()
@click.option("-p", default=settings.TPTP_ROOT)
def store_problems(p):
    settings.TPTP_ROOT = p
    build_tptp.store_problems()


@click.command()
@click.option("-p", default=settings.TPTP_ROOT)
def store_solutions(p):
    """Drop tables created gy init-db"""
    build_tptp.all_solution(p)


@click.command()
def drop_db():
    """Drop tables created gy init-db"""
    drop_tables()


@click.command()
@click.option("-p", default=settings.TPTP_ROOT)
def clear_db(p):
    """Drop tables created gy init-db and recreate them"""
    drop_tables()
    create_tables()


@click.command()
@click.argument("f")
@click.option("-s", default=None)
@click.option("--plot", is_flag=True, default=False)
def prove(f, s, plot):
    processor = TPTPParser()
    vp = VampireInterface()
    hp = HetsProve(vp)
    problems = list(processor.problem_processor(f))
    compiler = TPTPCompiler()
    for problem in problems:
        if s is not None:
            selector = Sine(
                premises=problem.premises, conjecture=problem.conjecture, max_depth=10
            )
            problem = Problem(premises=selector.select(), conjecture=problem.conjecture)
        proof = hp.prove(problem, compiler)
        if not plot:
            for s in proof.steps:
                print(
                    "{name}: {formula} [{source}]".format(
                        name=s.name, formula=s.formula, source=s.render_source()
                    )
                )
        else:
            g = proof.get_graph()
            g.render()


@click.command()
@click.argument("f")
def select(f):
    processor = TPTPParser()
    problem = list(processor.problem_processor(f))[0]
    selector = Sine(
        premises=problem.premises, conjecture=problem.conjecture, max_depth=10
    )
    smaller_problem = Problem(premises=selector.select(), conjecture=problem.conjecture)
    for prem in list(smaller_problem.premises):
        print(prem)
    print(smaller_problem.conjecture)


db.add_command(init_db)
db.add_command(drop_db)
db.add_command(clear_db)
db.add_command(store_problems)
db.add_command(store)
db.add_command(store_solutions)

db.add_command(prove)

db.add_command(select)

cli = click.CommandCollection(sources=[db])

main = cli

if __name__ == "__main__":
    cli()
